package service

import (
	"iot_manager_service/app/device/dao"
	"iot_manager_service/app/device/model"
	"iot_manager_service/app/system/service"
	"iot_manager_service/util/cache"
	"iot_manager_service/util/common"
	"iot_manager_service/util/logger"
	"time"
)

var OptoSensorService = new(optoSensorService)

type optoSensorService struct{}

func (s *optoSensorService) Get(id int) (*model.OptoSensorDetail, *common.Errors) {
	// 创建查询实例
	device := &dao.OptoSensor{
		ID: id,
	}
	err := device.GetDevice()
	if err != nil {
		return nil, common.FailResponse(err.Error(), nil)
	}
	endTime, state := cache.GetDeviceState(device.Sn)
	return &model.OptoSensorDetail{
		OptoSensor:   *device,
		RunState:     state,
		NetworkState: state,
		EndLineTime:  endTime,
	}, nil
}

func (s *optoSensorService) CreateOrUpdate(userId int64, tenantId int, req dao.OptoSensor) *common.Errors {
	// 创建查询实例
	device := req
	device.TenantId = tenantId
	device.UpdateUser = userId
	device.UpdateTime = time.Now()
	if req.ID == 0 {
		device.CreateTime = time.Now()
		device.CreateUser = userId
		if device.IsExistedBySN() {
			logger.Logger.Errorf("Create GetDeviceID err \n")
			return common.ParamsInvalidResponse("编码不能重复，请重新填写！", nil)
		}
		if err := device.Create(); err != nil {
			logger.Logger.Errorf("Create err = %s \n", err.Error())
			return common.FailResponse(err.Error(), nil)
		}
		service.OperationHisService.Save(userId, tenantId, common.OperationCreate, common.ModuleTypeDevice,
			common.DeviceTypeOptoSensor, common.GetDeviceObject(device.ID, device.Name), common.OperationSuccess)
		return common.SuccessResponse(common.Succeeded, nil)
	}
	if device.IsExistedByNameAndCode() {
		return common.ParamsInvalidResponse("编码不能重复，请重新填写！", nil)
	}

	if err := device.Update(); err != nil {
		logger.Logger.Errorf("Update err = %s \n", err.Error())
		return common.FailResponse(err.Error(), nil)
	}

	service.OperationHisService.Save(userId, tenantId, common.OperationUpdate, common.ModuleTypeDevice,
		common.DeviceTypeOptoSensor, common.GetDeviceObject(device.ID, device.Name), common.OperationSuccess)
	return common.SuccessResponse(common.Succeeded, nil)
}

func (s *optoSensorService) List(searchValue string, current, size int) ([]model.OptoSensorDetail, *common.Errors) {
	device := dao.OptoSensor{}
	if searchValue != "" {
		device.Sn = searchValue
		device.Name = searchValue
	}

	offset := (current - 1) * size
	limit := size
	devices, err := device.GetDevices(offset, limit)
	if err != nil {
		return nil, common.FailResponse(err.Error(), nil)
	}
	var details []model.OptoSensorDetail
	for _, d := range devices {
		endTime, state := cache.GetDeviceState(d.Sn)
		details = append(details, model.OptoSensorDetail{
			OptoSensor:   d,
			RunState:     state,
			NetworkState: state,
			EndLineTime:  endTime,
		})
	}
	return details, nil
}

func (s *optoSensorService) Remove(userId int64, tenantId int, id int) *common.Errors {
	// 创建查询实例
	device := &dao.OptoSensor{
		ID:         id,
		IsDeleted:  1,
		UpdateUser: userId,
		UpdateTime: time.Now(),
	}
	err := device.Delete()
	if err != nil {
		return common.FailResponse(err.Error(), nil)
	}
	service.OperationHisService.Save(userId, tenantId, common.OperationRemove, common.ModuleTypeDevice,
		common.DeviceTypeOptoSensor, common.GetDeviceObject(device.ID, device.Name), common.OperationSuccess)
	return nil
}

func (s *optoSensorService) GetList(tenantId int) ([]*dao.OptoSensor, *common.Errors) {
	device := &dao.OptoSensor{
		TenantId:  tenantId,
		IsDeleted: 0,
	}
	devices, err := device.GetAllDevices()
	if err != nil {
		return nil, common.FailResponse(err.Error(), nil)
	}

	return devices, nil
}

func (s *optoSensorService) GetByGateway(id int) []dao.OptoSensor {
	// 创建查询实例
	device := &dao.OptoSensor{
		GatewayId: id,
	}
	return device.GetDevicesByGateway()
}

func (s *optoSensorService) GetByLampPole(id int) []dao.OptoSensor {
	// 创建查询实例
	device := &dao.OptoSensor{
		LampPoleId: id,
	}
	return device.GetDevicesByLampPole()
}

func (s *optoSensorService) GetByTenant(id int) []dao.OptoSensor {
	// 创建查询实例
	device := &dao.OptoSensor{
		TenantId: id,
	}
	return device.GetDevicesByTenantId()
}

func (s *optoSensorService) GetOne(id int) *dao.OptoSensor {
	// 创建查询实例
	device := &dao.OptoSensor{
		ID: id,
	}
	err := device.GetDevice()
	if err != nil {
		return nil
	}
	return device
}

func (s *optoSensorService) GetAll() []*dao.OptoSensor {
	device := &dao.OptoSensor{}
	devices := device.GetAll()
	return devices
}
